@@ -2,11 +2,11 @@
 # encoding: utf-8
 
 __author__ = "ChenyangGao <https://chenyanggao.github.io>"
-__version__ = (0, 0, 7)
+__version__ = (0, 0, 9)
 __all__ = [
     "thread_batch", "thread_pool_batch", "async_batch", 
     "threaded", "run_as_thread", "asynchronized", "run_as_async", 
-    "threadpool_map", "taskgroup_map", 
+    "threadpool_map", "taskgroup_map", "conmap", "Return", 
 ]
 
 from asyncio import (
@@ -19,16 +19,15 @@ from collections.abc import (
 )
 from concurrent.futures import Executor, Future, ThreadPoolExecutor
 from functools import partial, update_wrapper
-from inspect import isawaitable, iscoroutinefunction
-from itertools import takewhile
+from inspect import isawaitable, iscoroutinefunction, signature
 from os import cpu_count
 from queue import Queue, SimpleQueue, Empty
 from _thread import start_new_thread
 from threading import Event, Lock, Semaphore, Thread
-from typing import cast, Any, ContextManager, Literal
+from typing import cast, overload, Any, ContextManager, Literal
 
 from argtools import argcount
-from asynctools import async_zip, ensure_coroutine, run_async
+from asynctools import async_zip, run_async
 from decotools import optional
 
 
@@ -38,6 +37,11 @@ if "__del__" not in TaskGroup.__dict__:
     setattr(TaskGroup, "__del__", lambda self, /: run_async(self.__aexit__(None, None, None)))
 
 
+class Return:
+    def __init__(self, value, /):
+        self.value = value
+
+
 def thread_batch[T, V](
     work: Callable[[T], V] | Callable[[T, Callable], V], 
     tasks: Iterable[T], 
@@ -275,11 +279,10 @@ def threadpool_map[T](
     it, 
     /, 
     *its, 
+    arg_func: None | Callable = None, 
     max_workers: None | int = None, 
-    kwargs: None | Mapping = None, 
+    kwargs: Mapping = {}, 
 ) -> Iterator[T]:
-    if kwargs is None:
-        kwargs = {}
     if max_workers is None or max_workers <= 0:
         max_workers = min(32, (cpu_count() or 1) + 4)
     queue: SimpleQueue[None | Future] = SimpleQueue()
@@ -289,9 +292,18 @@ def threadpool_map[T](
     running = True
     def make_tasks():
         try:
-            for args in takewhile(lambda _: running, zip(it, *its)):
+            for args in zip(it, *its):
+                if not running:
+                    break
                 try:
-                    put(submit(func, *args, **kwargs))
+                    if arg_func is None:
+                        put(submit(func, *args, **kwargs))
+                    else:
+                        arg = arg_func(*args, **kwargs)
+                        if isinstance(arg, Return):
+                            put(arg.value)
+                            continue
+                        put(submit(func, *args, arg, **kwargs))
                 except RuntimeError:
                     break
         finally:
@@ -311,13 +323,12 @@ async def taskgroup_map[T](
     it, 
     /, 
     *its, 
+    arg_func: None | Callable = None, 
     max_workers: None | int = None, 
-    kwargs: None | Mapping = None, 
+    kwargs: Mapping = {}, 
 ) -> AsyncIterator[T]:
     if max_workers is None or max_workers <= 0:
         max_workers = 32
-    if kwargs is None:
-        kwargs = {}
     sema = AsyncSemaphore(max_workers)
     queue: AsyncQueue[None | AsyncFuture] = AsyncQueue()
     get, put = queue.get, queue.put_nowait
@@ -327,6 +338,14 @@ async def taskgroup_map[T](
     async def make_tasks():
         try:
             async for args in async_zip(it, *its):
+                if arg_func is not None:
+                    arg = arg_func(*args, **kwargs)
+                    if isawaitable(arg):
+                        arg = await arg
+                    if isinstance(arg, Return):
+                        put(arg.value)
+                        continue
+                    args = (*args, arg)
                 put(create_task(call(args)))
         finally:
             put(None)
@@ -336,3 +355,52 @@ async def taskgroup_map[T](
         while task := await get():
             yield await task
 
+
+@overload
+def conmap[T](
+    func: Callable[..., T], 
+    it, 
+    /, 
+    *its, 
+    arg_func: None | Callable = None, 
+    max_workers: None | int = None, 
+    kwargs: Mapping = {}, 
+    async_: Literal[False] = False, 
+) -> Iterator[T]:
+    ...
+@overload
+def conmap[T](
+    func: Callable[..., Awaitable[T]], 
+    it, 
+    /, 
+    *its, 
+    arg_func: None | Callable = None, 
+    max_workers: None | int = None, 
+    kwargs: Mapping = {}, 
+    async_: Literal[True], 
+) -> AsyncIterator[T]:
+    ...
+def conmap[T](
+    func: Callable[..., T] | Callable[..., Awaitable[T]], 
+    it, 
+    /, 
+    *its, 
+    arg_func: None | Callable = None, 
+    max_workers: None | int = None, 
+    kwargs: Mapping = {}, 
+    async_: Literal[False, True] = False, 
+) -> Iterator[T] | AsyncIterator[T]:
+    map = taskgroup_map if async_ else threadpool_map
+    params = signature(func).parameters
+    if ((param := params.get("async_")) and 
+        (param.kind in (param.POSITIONAL_OR_KEYWORD, param.KEYWORD_ONLY))
+    ):
+        kwargs = {"async_": async_, **kwargs}
+    return map(
+        it, 
+        *its, 
+        arg_func=arg_func, 
+        max_workers=max_workers, 
+        kwargs=kwargs, 
+    )
+
